#
# Copyright (c) 2013 Dorian Pula <dorian.pula@amber-penguin-software.ca>
#
# Rookeries is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of
# the License, or (at your option) any later version.
#
# Rookeries is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public
# License along with Rookeries.  If not, see <http://www.gnu.org/licenses/>.
#
# Please share and enjoy!
#

"""
Core module that handles security in Rookeri.es

:copyright: Copyright 2013, Dorian Pula <dorian.pula@amber-penguin-software.ca>
:license: AGPL v3+
"""

# TODO Clean this up!!!

# TODO Add in some sort of checking of user authentication if the user is allowed to login.
# TODO Add in check to see if user is part of group that can view content.

import random
import string
import uuid
from datetime import datetime, timedelta

from sqlalchemy.orm.exc import NoResultFound

from rookeries.users.models import ActivationKey, User, UserAuthentication, UserStatus, UserAuthenticationToken
from rookeries.core.email import email_message
from rookeries.security import authentication


def change_password(config, user, password):
    """Generate the password hash and change it in the object."""

    user.user_authentication.security_update_date = datetime.now()
    hashed_password = authentication.generate_password_hash(config, user, password)
    user.user_authentication.password = hashed_password


def check_user_login(config, user, password):
    """Checks to see if the user can login."""

    if authentication.check_password(config, user, password):
        return user.user_authentication.status == UserStatus.active

    else:
        return False


def update_user_state(user, new_state, db_session):
    """Updates the state of the user to a new state."""

    current_state = user.user_authentication.status

    if (current_state == UserStatus.created) and (new_state == UserStatus.active):

        # Remove the remaining activation keys.
        user.user_authentication.status = new_state
        used_activations = db_session.query(ActivationKey).filter(ActivationKey.user == user).all()
        for activation in used_activations:
            db_session.delete(activation)
        db_session.commit()

    # TODO Add other states as well.


def email_user_activation_code(app, user, key):
    """Emails activation codes to the user."""

    message = "Navigate to: http://rookeri.es/activate/%s to activate your user account." % key

    email_message(app, subject="Rookeri.es User Activation Code", sender=app.config["MAIL_DEFAULT_SENDER"],
                  recipient=user.email, message=message)


def create_new_activation_code(db_session, user):
    """Creates a new authentication code."""

    key_in_use = True
    while key_in_use:

        key = ""
        for i in xrange(128):
            # TODO Use uuids instead of crafting your own random string?
            key += random.choice(string.digits + string.ascii_letters)

        url_ready_key = unicode(key)

        existing_key = db_session.query(ActivationKey).filter(ActivationKey.key == url_ready_key).first()
        key_in_use = existing_key is not None

    expiry_date = datetime.now() + timedelta(days=2)
    activation_key = ActivationKey(user, url_ready_key, created_date=datetime.now(), expiry_date=expiry_date)
    db_session.add(activation_key)

    return activation_key


# TODO Add in expiration and removal of activation keys and authentication tokens.

def create_new_user(app, db_session, email, password, user_full_name, username):
    """Creates a new user and sends out an activation email."""

    user = User(username=username, full_name=user_full_name, email=email)

    user.user_authentication = UserAuthentication()
    user.user_authentication.status = UserStatus.created
    hashed_password = authentication.generate_password_hash(app.config, user, password)
    user.user_authentication.password = hashed_password

    # Create the activation code and email out the invite.
    activation_code = create_new_activation_code(db_session, user)

    # TODO Handle issues where username or email already exists.
    db_session.add(user)
    db_session.commit()

    email_user_activation_code(app, user, activation_code.key)
    return user


def create_auth_token(db_session, user):
    """Creates a new authentication token."""

    token = uuid.uuid4()
    created_date = datetime.now()
    expiry_date = datetime.now() + timedelta(days=2)

    auth_token = UserAuthenticationToken(user, token, created_date=created_date, expiry_date=expiry_date)
    db_session.add(auth_token)

    return auth_token


def remove_auth_token(db_session, user, token):
    """Removes an existing authentication token."""

    auth_token = db_session.query(UserAuthenticationToken).filter(UserAuthenticationToken.user == user)\
        .filter(UserAuthenticationToken.token == token).first()

    db_session.delete(auth_token)
    db_session.commit()


def validate_auth_token(db_session, user, challenge_token):
    """Validates a given authentication token."""

    valid_token = True

    try:
        auth_token = db_session.query(UserAuthenticationToken).filter(UserAuthenticationToken.user == user)\
            .filter(UserAuthenticationToken.token == challenge_token).one()
    except NoResultFound:
        valid_token = False

    if valid_token:
        current_time = datetime.now()
        if (auth_token.created_date <= current_time) and (auth_token.expiry_date > current_time):
            valid_token = True
        else:
            db_session.delete(auth_token)
            db_session.commit()
            valid_token = False

    return valid_token
