#
# Copyright (c) 2013 Dorian Pula <dorian.pula@amber-penguin-software.ca>
#
# Rookeries is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of
# the License, or (at your option) any later version.
#
# Rookeries is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public
# License along with Rookeries.  If not, see <http://www.gnu.org/licenses/>.
#
# Please share and enjoy!
#

"""
Models for the core of the Rookeries platform.

:copyright: Copyright 2013, Dorian Pula <dorian.pula@amber-penguin-software.ca>
:license: AGPL v3+
"""

from datetime import datetime

from sqlalchemy import Column, Integer, String, DateTime, ForeignKey, Table
from sqlalchemy.orm import relationship

from rookeries.core import database
from rookeries.core.sqlalchemy_enums import DeclEnum

# TODO Add in user preferences.  Do as part of user profile detail?


class User(database.db.Model):
    """User of the Rookeries application."""

    __tablename__ = "user"

    id = Column(Integer, primary_key=True)
    username = Column(String(48), unique=True)
    full_name = Column(String(256))
    email = Column(String(128), unique=True)

    user_auth_id = Column(Integer, ForeignKey("user_authentication.id"))
    user_authentication = relationship("UserAuthentication", backref="user", cascade="all, delete")

    def __init__(self, username=None, full_name=None, email=None):
        self.username = username
        self.full_name = full_name
        self.email = email

    def __repr__(self):
        return "<User('%s', '%s', '%s')>" % (self.username, self.email, self.full_name)


class UserStatus(DeclEnum):
    """Posible user statuses."""
    created = "created", "Created"
    active = "active", "Active"
    disabled = "disabled", "Disabled"
    banned = "banned", "Banned"


class UserAuthentication(database.db.Model):
    """Model for storing and managing user authentication."""

    __tablename__ = "user_authentication"

    id = Column(Integer, primary_key=True)

    password = Column(String(512))
    security_update_date = Column(DateTime)
    status = Column(UserStatus.db_type())

    def __init__(self, status=UserStatus.created):
        self.status = status
        self.security_update_date = datetime.now()

    def __repr__(self):
        return "<UserAuthentication('%s', '%s', '%s')>" % (self.id, self.status, self.security_update_date)


class UserProfileDetailType(DeclEnum):
    """Possible user profile details types."""
    # TODO Consider something a bit more flexible for the myriad of web services that are out there?
    email = "Email"
    address = "Address"
    city = "City"
    region = "Region"
    country = "Country"
    cell_phone = "CellPhone"
    phone = "Phone"
    microblog = "Microblog"
    url = "Url"


class UserProfileDetail(database.db.Model):
    """Flexible elements for storing elements of a user profile."""

    __tablename__ = "user_profile_details"

    id = Column(Integer, primary_key=True)
    user_id = Column(Integer, ForeignKey("user.id"))
    user = relationship("User")
    detail_type = Column(UserProfileDetailType.db_type())
    detail_value = Column(String(512))

    def __init__(self, user, detail_type=UserProfileDetailType.email, detail_value=None):
        self.user = user
        self.detail_type = detail_type
        self.detail_value = detail_value

    def __repr__(self):
        return "<UserProfileDetails('%s', '%s', '%s', '%s')>" % (self.id, self.user_id, self.detail_type,
                                                                 self.detail_value)

user_group_membership_table = Table("user_group_membership", database.db.Model.metadata,
                                    Column("user_id", Integer, ForeignKey("user.id")),
                                    Column("group_id", Integer, ForeignKey("group.id")))


class Group(database.db.Model):
    """Group of users that allows for controlling user interactions and permissions in a granular manner."""

    __tablename__ = "group"

    id = Column(Integer, primary_key=True)
    name = Column(String(512))
    parent_group_id = Column(Integer, ForeignKey("group.id"), nullable=True)
    parent_group = relationship("Group")
    members = relationship("User", secondary=user_group_membership_table)

    def __init__(self, name, parent=None, user_members=None):
        self.name = name
        self.parent_group = parent
        self.members = user_members

    def __repr__(self):
        return "<Group('%s', '%s', '%s')>" % (self.id, self.name, self.parent_group.name)


class ActivationKey(database.db.Model):
    """Activation key for users that have registered with the site."""

    __tablename__ = "activation_key"

    id = Column(Integer, primary_key=True)
    user_id = Column(Integer, ForeignKey("user.id"))
    user = relationship("User")

    key = Column(String(128), unique=True)
    created_date = Column(DateTime)
    expiry_date = Column(DateTime)

    def __init__(self, user, key, created_date=datetime.now(), expiry_date=datetime.now()):
        self.user = user
        self.key = key
        self.created_date = created_date
        self.expiry_date = expiry_date

    def __repr__(self):
        return "<ActivationKey('%s', '%s', '%s-%s')>" % (self.id, self.user.username, self.created_date,
                                                         self.expiry_date)


class UserAuthenticationToken(database.db.Model):
    """Authentication tokens for a user."""

    __tablename__ = "user_authentication_token"

    id = Column(Integer, primary_key=True)
    user_id = Column(Integer, ForeignKey("user.id"))
    user = relationship("User")

    token = Column(String(128), unique=True)
    created_date = Column(DateTime)
    expiry_date = Column(DateTime)

    def __init__(self, user, token, created_date=datetime.now(), expiry_date=datetime.now()):
        self.user = user
        self.token = token
        self.created_date = created_date
        self.expiry_date = expiry_date

    def __repr__(self):
        return "<ActivationKey('%s', '%s', '%s-%s')>" % (self.id, self.user.username, self.created_date,
                                                         self.expiry_date)
