#
# Copyright (c) 2013-2014 Dorian Pula <dorian.pula@amber-penguin-software.ca>
#
# Rookeries is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of
# the License, or (at your option) any later version.
#
# Rookeries is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public
# License along with Rookeries.  If not, see <http://www.gnu.org/licenses/>.
#
# Please share and enjoy!
#

import os
import unittest

from nose import tools as asserts

import rookeries
from rookeries import config


class TestConfiguration(unittest.TestCase):

    def setUp(self):
        self.test_app_client = rookeries.make_rookeries_app().test_client()
        self.test_app = self.test_app_client.application
        self.test_app_config = self.test_app_client.application.config

    def test_configuration_will_use_default_configuration_class(self):
        config.configure_web_app(self.test_app)
        expected_config = config.DefaultConfig()

        for key, value in expected_config.__dict__.iteritems():
            asserts.assert_in(key, self.test_app_config)
            asserts.assert_equal(value, self.test_app_config[key])

    def test_configuration_will_use_envs_with_appropriate_prefix(self):
        test_config_key = 'CONFIG_KEY'
        test_config_value = 'ROOKERIES_CONFIG_VALUE'

        with set_config_in_environ(test_config_key, test_config_value):
            config.configure_web_app(self.test_app)
            asserts.assert_in(test_config_key, self.test_app_config)
            asserts.assert_equal(test_config_value, self.test_app_config[test_config_key])

    def test_configuration_will_skip_envs_with_wrong_prefix(self):
        test_config_key = 'CONFIG_KEY'
        test_config_value = 'BAD_CONFIG_VALUE'

        with set_config_in_environ(test_config_key, test_config_value, prefix='RANDOM_'):
            config.configure_web_app(self.test_app)
            asserts.assert_not_in(test_config_key, self.test_app_config)

    def test_configuration_from_environment_variable_will_override_default(self):
        test_config_key = 'ENVIRONMENTAL_NAME'
        test_config_value = 'test_environ'

        with set_config_in_environ(test_config_key, test_config_value):
            config.configure_web_app(self.test_app)
            asserts.assert_in(test_config_key, self.test_app_config)
            asserts.assert_not_equal(config.DefaultConfig.ENVIRONMENTAL_NAME, self.test_app_config[test_config_key])
            asserts.assert_equal(test_config_value, self.test_app_config[test_config_key])


class set_config_in_environ(object):
    """Context manager to change an environment variable."""

    def __init__(self, environ_variable, environ_value, prefix=config.ROOKERIES_ENV_CONFIG_PREFIX):
        self.environ_name = ''.join((prefix, environ_variable))
        self.new_environ_value = environ_value
        self.old_environ_value = os.environ.get('config_key')

    def __enter__(self):
        os.environ[self.environ_name] = self.new_environ_value

    def __exit__(self, etype, value, traceback):
        os.environ.pop(self.environ_name)
        if self.old_environ_value:
            os.environ[self.environ_name] = self.old_environ_value
